/*
 * DowncastRegistry.hpp
 *
 *  Created on: 11. 7. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _DOWNCAST_REGISTRY_HPP_
#define _DOWNCAST_REGISTRY_HPP_

#include <memory>
#include <string>
#include <map>

#include <exception/CommonException.h>
#include <abstraction/OperationAbstraction.hpp>

namespace abstraction {

class DowncastRegistry {
	class Entry {
	public:
		virtual std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( ) const = 0;

	};

	template < class ConcreteType, class BaseType >
	class EntryImpl : public Entry {
	public:
		EntryImpl ( ) {
		}

		virtual std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( ) const override;
	};

	static ext::map < std::pair < std::string, std::string >, std::unique_ptr < Entry > > & getEntries ( ) {
		static ext::map < std::pair < std::string, std::string >, std::unique_ptr < Entry > > fileWriters;
		return fileWriters;
	}

public:
	template < class ConcreteType, class BaseType >
	static void registerDowncast ( ) {
		std::string concrete = ext::to_string < ConcreteType > ( );
		std::string base = ext::to_string < BaseType > ( );
		if ( ! getEntries ( ).insert ( std::make_pair ( std::make_pair ( concrete, base ), std::unique_ptr < Entry > ( new EntryImpl < ConcreteType, BaseType > ( ) ) ) ).second )
			throw ::exception::CommonException ( "Downcasting for " + base + " to " + concrete + " already registered." );
	}

	static std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( const std::string & concrete, const std::string & base );
};

} /* namespace abstraction */

#include <abstraction/DowncastAbstraction.hpp>

namespace abstraction {

template < class ConcreteType, class BaseType >
std::shared_ptr < abstraction::OperationAbstraction > DowncastRegistry::EntryImpl < ConcreteType, BaseType >::getAbstraction ( ) const {
	return std::make_shared < DowncastAbstraction < ConcreteType, const BaseType & > > ( );
}

} /* namespace abstraction */

#endif /* _DOWNCAST_REGISTRY_HPP_ */
