/*
 * NormalizeRegistry.hpp
 *
 *  Created on: 11. 7. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _NORMALIZE_REGISTRY_HPP_
#define _NORMALIZE_REGISTRY_HPP_

#include <memory>
#include <string>
#include <map>

#include <exception/CommonException.h>
#include <abstraction/OperationAbstraction.hpp>

namespace abstraction {

class NormalizeRegistry {
	class Entry {
	public:
		virtual std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( ) const = 0;

	};

	template < class Param >
	class EntryImpl : public Entry {
	public:
		EntryImpl ( ) {
		}

		virtual std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( ) const override;
	};

	static ext::map < std::string, std::unique_ptr < Entry > > & getEntries ( ) {
		static ext::map < std::string, std::unique_ptr < Entry > > fileWriters;
		return fileWriters;
	}

public:
	template < class ParamType >
	static void registerNormalize ( std::string param ) {
		getEntries ( ).insert ( std::make_pair ( param, std::unique_ptr < Entry > ( new EntryImpl < ParamType > ( ) ) ) );
	}

	template < class ParamType >
	static void registerNormalize ( ) {
		std::string param = ext::to_string < ParamType > ( );
		registerNormalize < ParamType > ( std::move ( param ) );
	}

	static std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( const std::string & param );
};

} /* namespace abstraction */

#include <abstraction/NormalizeAbstraction.hpp>

namespace abstraction {

template < class Param >
std::shared_ptr < abstraction::OperationAbstraction > NormalizeRegistry::EntryImpl < Param >::getAbstraction ( ) const {
	return std::make_shared < NormalizeAbstraction < typename Param::normalized_type, Param > > ( );
}

} /* namespace abstraction */

#endif /* _NORMALIZE_REGISTRY_HPP_ */
