/*
 * OperationAbstraction.hpp
 *
 *  Created on: 10. 7. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _OPERATION_ABSTRACTION_HPP_
#define _OPERATION_ABSTRACTION_HPP_

#include <memory>
#include <string>
#include <tuple>
#include <functional>

#include <exception/CommonException.h>

namespace abstraction {

class OperationAbstraction : public std::enable_shared_from_this < OperationAbstraction > {
public:
	virtual bool attachInput ( const std::shared_ptr < OperationAbstraction > & input, unsigned index, bool move ) = 0;
	virtual bool detachInput ( unsigned index ) = 0;

	virtual ~OperationAbstraction ( ) noexcept {
	}

	virtual bool inputsReady ( ) const = 0;
	virtual bool inputsAttached ( ) const = 0;
	virtual bool run ( ) = 0;
	virtual bool eval ( ) = 0;
	virtual unsigned numberOfParams ( ) const = 0;
	virtual bool cached ( ) const = 0;

	virtual ext::type_index getParamTypeIndex ( unsigned index ) const = 0;
	virtual ext::type_index getReturnTypeIndex ( ) const = 0;
	virtual ext::type_index getRuntimeReturnTypeIndex ( ) const = 0;

	std::string getParamType ( unsigned index ) const {
		return ext::to_string ( getParamTypeIndex ( index ) );
	}

	std::string getReturnType ( ) const {
		return ext::to_string ( getReturnTypeIndex ( ) );
	}

	std::string getRuntimeReturnType ( ) const {
		return ext::to_string ( getRuntimeReturnTypeIndex ( ) );
	}

};

} /* namespace abstraction */

#endif /* _OPERATION_ABSTRACTION_HPP_ */
