/*
 * CommonException.cpp
 *
 * Created on: Apr 1, 2013
 * Author: Martin Zak
 */

#include "CommonException.h"

#include "../sax/FromXMLParserHelper.h"
#include "../object/Object.h"
#include "../core/xmlApi.hpp"
#include "../object/UniqueObject.h"
#include "../global/GlobalData.h"

#include <registration/ValuePrinterRegistration.hpp>
#include <registration/XmlRegistration.hpp>

#ifdef DEBUG
	#include "../debug/simpleStacktrace.h"
#endif

namespace exception {

CommonException::CommonException ( std::string cause ) : m_cause(std::move(cause)) {
	#ifdef DEBUG
		this->m_backtrace = ext::simpleStacktrace();
	#else
		this->m_backtrace = "";
	#endif

	for(int i = 0; i < common::GlobalData::argc; i++) {
		if(i != 0) this->m_command += " ";
		this->m_command += common::GlobalData::argv[i];
	}
	this->m_whatMessage += this->m_backtrace;
	this->m_whatMessage += this->m_cause;
}

CommonException::CommonException ( std::string cause, std::string backtrace, std::string command ) : m_cause(std::move(cause)), m_backtrace(std::move(backtrace)), m_command(std::move(command)) {
	this->m_whatMessage += this->m_backtrace;
	this->m_whatMessage += this->m_cause;
}

alib::ObjectBase* CommonException::clone() const {
	return new CommonException(*this);
}

alib::ObjectBase* CommonException::plunder() && {
	return new CommonException(std::move(*this));
}

int CommonException::compare(const CommonException& other) const {
	return this->m_whatMessage.compare(other.m_whatMessage);
}

const char * CommonException::what ( ) const noexcept {
	return m_whatMessage.c_str ( );
}

const std::string & CommonException::getCause ( ) const {
	return m_cause;
}

const std::string & CommonException::getBacktrace ( ) const {
	return m_backtrace;
}

const std::string & CommonException::getCommand ( ) const {
	return m_command;
}

void CommonException::operator>>(std::ostream& os) const {
	os << this->m_whatMessage;
}

CommonException::operator std::string() const {
	return this->m_whatMessage;
}

alib::ObjectBase* CommonException::inc() && {
	return new alib::UniqueObject(alib::Object(std::move(*this)), primitive::Integer(0));
}

} /* namespace exception */

namespace alib {

exception::CommonException xmlApi < exception::CommonException >::parse ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, xmlTagName());
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "cause");
	std::string cause = "";
	if (input->getType() == sax::Token::TokenType::CHARACTER) {
		cause = std::move(*input).moveData();
		++input;
	}
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "cause");
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "backtrace");
	std::string backtrace = "";
	if (input->getType() == sax::Token::TokenType::CHARACTER) {
		backtrace = std::move(*input).moveData();
		++input;
	}
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "backtrace");
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "command");
	std::string command = "";
	if (input->getType() == sax::Token::TokenType::CHARACTER) {
		command = std::move(*input).moveData();
		++input;
	}
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "command");
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, xmlTagName());
	return exception::CommonException(std::move(cause), std::move(backtrace), std::move(command));
}

bool xmlApi < exception::CommonException >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

const std::string & xmlApi < exception::CommonException >::xmlTagName ( ) {
	static std::string xmlTagName = "CommonException";

	return xmlTagName;
}

void xmlApi < exception::CommonException >::compose ( ext::deque < sax::Token > & output, const exception::CommonException & data ) {
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::START_ELEMENT );
	output.emplace_back ( "cause", sax::Token::TokenType::START_ELEMENT );
	output.emplace_back ( data.getCause ( ), sax::Token::TokenType::CHARACTER );
	output.emplace_back ( "cause", sax::Token::TokenType::END_ELEMENT );
	output.emplace_back ( "backtrace", sax::Token::TokenType::START_ELEMENT );
	output.emplace_back ( data.getBacktrace ( ), sax::Token::TokenType::CHARACTER );
	output.emplace_back ( "backtrace", sax::Token::TokenType::END_ELEMENT );
	output.emplace_back ( "command", sax::Token::TokenType::START_ELEMENT );
	output.emplace_back ( data.getCommand ( ), sax::Token::TokenType::CHARACTER );
	output.emplace_back ( "command", sax::Token::TokenType::END_ELEMENT );
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::END_ELEMENT );
}

} /* namespace alib */

namespace {

static auto valuePrinter = registration::ValuePrinterRegister < exception::CommonException > ( );

static auto xmlWrite = registration::XmlWriterRegister < alib::Object, exception::CommonException > ( );

static auto xmlReaded = registration::XmlReaderRegister < alib::Object, exception::CommonException > ( );

} /* namespace */
