/*
 * AnyObject.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef ANY_OBJECT_H_
#define ANY_OBJECT_H_

#include "ObjectBase.h"
#include <deque>
#include "../sax/Token.h"

#include "../sax/FromXMLParserHelper.h"
#include "Object.h"
#include "../core/xmlApi.hpp"
#include "UniqueObject.h"

namespace alib {

/**
 * Represents void.
 */
template < class T >
class AnyObject final : public ObjectBase {
	T m_data;
public:
	/**
	 * Creates a blank symbol.
	 * @param symbol name of the symbol
	 */
	explicit AnyObject ( const T & data );
	explicit AnyObject ( T && data );

	virtual ObjectBase * clone ( ) const;
	virtual ObjectBase * plunder ( ) &&;

	const T & getData ( ) const;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const AnyObject & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	static void compose ( ext::deque < sax::Token > & out, const AnyObject & input );

	virtual ObjectBase * inc ( ) &&;
};

template < class T >
AnyObject < T >::AnyObject ( const T & data ) : m_data ( data ) {
}

template < class T >
AnyObject < T >::AnyObject ( T && data ) : m_data ( std::move ( data ) ) {
}

template < class T >
ObjectBase* AnyObject < T >::clone() const {
	return new AnyObject(*this);
}

template < class T >
ObjectBase* AnyObject < T >::plunder() && {
	return new AnyObject(std::move(*this));
}

template < class T >
const T & AnyObject < T >::getData ( ) const {
	return m_data;
}

template < class T >
int AnyObject < T >::compare(const AnyObject < T > & other ) const {
	static ext::compare < decltype ( m_data ) > comp;

	return comp ( m_data, other.m_data );
}

template < class T >
void AnyObject < T >::operator>>(std::ostream& out) const {
	out << "(AnyObject" << m_data << ")";
}

template < class T >
AnyObject < T > ::operator std::string () const {
	return "V";
}

template < class T >
ObjectBase* AnyObject < T >::inc() && {
	return new UniqueObject(Object(std::move(*this)), primitive::Integer(0));
}

} /* namespace alib */

namespace alib {

template < typename T >
struct xmlApi < alib::AnyObject < T > > {
	static alib::AnyObject < T > parse ( ext::deque < sax::Token >::iterator & input );
	static bool first ( const ext::deque < sax::Token >::const_iterator & input );
	static const std::string & xmlTagName ( );
	static void compose ( ext::deque < sax::Token > & output, const alib::AnyObject < T > & data );
};

template < typename T >
alib::AnyObject < T > xmlApi < alib::AnyObject < T > >::parse ( ext::deque < sax::Token >::iterator & input ) {
	return alib::AnyObject < T > ( alib::xmlApi < T >::parse ( input ) );
}

template < typename T >
bool xmlApi < alib::AnyObject < T > >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return alib::xmlApi < T >::first ( input );
}

template < typename T >
const std::string & xmlApi < alib::AnyObject < T > >::xmlTagName ( ) {
	return alib::xmlApi < T >::xmlTagName ( );
}

template < typename T >
void xmlApi < alib::AnyObject < T > >::compose ( ext::deque < sax::Token > & output, const alib::AnyObject < T > & input ) {
	alib::xmlApi < T >::compose ( output, input.getData ( ) );
}

} /* namespace alib */

#endif /* ANY_OBJECT_H_ */
