/*
 * ObjectBase.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef OBJECT_BASE_H_
#define OBJECT_BASE_H_

#include <deque>
#include <memory>
#include "../sax/Token.h"
#include "../base/CommonBase.hpp"

namespace alib {

/**
 * Represents a base for any object.
 */
class ObjectBase : public alib::CommonBase<ObjectBase>, public ext::cow_shared_ptr_base {
public:
	/**
	 * @copydoc alib::CommonBase<ObjectBase>::clone()
	 */
	virtual ObjectBase * clone() const override = 0;

	/**
	 * @copydoc alib::CommonBase<ObjectBase>::plunder()
	 */
	virtual ObjectBase * plunder() && override = 0;

	/**
	 * \brief Similar to operator ++.
	 *
	 * \details Effectively may discard or preserve the instance based on the way how the incrementation can be done.
	 */
	virtual ObjectBase * inc() && = 0;
};

} /* namespace alib */

#endif /* OBJECT_BASE_H_ */

