/*
 * UniqueObject.cpp
 *
 *  Created on: Nov 11, 2016
 *      Author: Jan Travicek
 */

#include "UniqueObject.h"
#include <sstream>
#include <sax/FromXMLParserHelper.h>

#include <registration/ValuePrinterRegistration.hpp>
#include <registration/XmlRegistration.hpp>

namespace alib {

UniqueObject::UniqueObject(Object symbol, primitive::Integer id) : m_symbol(std::move(symbol)), m_id(std::move(id)) {

}

ObjectBase* UniqueObject::clone() const {
	return new UniqueObject(*this);
}

ObjectBase* UniqueObject::plunder() && {
	return new UniqueObject(std::move(*this));
}

const Object& UniqueObject::getSymbol() const {
	return m_symbol;
}

const primitive::Integer& UniqueObject::getId() const {
	return m_id;
}

primitive::Integer& UniqueObject::getId() {
	return m_id;
}

int UniqueObject::compare(const UniqueObject& other) const {
	int res = m_symbol.getData().compare(other.m_symbol.getData());
	if(res == 0) res = m_id.compare(other.m_id);
	return res;
}

void UniqueObject::operator>>(std::ostream& out) const {
	out << "(UniqueObject " << m_symbol << ", " << m_id << ")";
}

UniqueObject::operator std::string() const {
	std::stringstream ss;
	ss << "{";
	ss << m_symbol;
	ss << ", ";
	ss << m_id;
	ss << "}";
	return std::move(ss).str();
}

ObjectBase* UniqueObject::inc() && {
	std::move(m_id).inc();
	return NULL;
}

} /* namespace alib */

namespace alib {

alib::UniqueObject xmlApi < alib::UniqueObject >::parse ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );

	Object firstSymbol = alib::xmlApi < Object >::parse ( input );
	primitive::Integer secondSymbol = alib::xmlApi < primitive::Integer >::parse ( input );

	UniqueObject data ( std::move ( firstSymbol ), std::move ( secondSymbol ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, xmlTagName ( ) );
	return data;
}

bool xmlApi < alib::UniqueObject >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

const std::string & xmlApi < alib::UniqueObject >::xmlTagName ( ) {
	static std::string xmlTagName = "UniqueObject";

	return xmlTagName;
}

void xmlApi < alib::UniqueObject >::compose ( ext::deque < sax::Token > & output, const alib::UniqueObject & object ) {
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::START_ELEMENT );
	alib::xmlApi < alib::Object >::compose ( output, object.getSymbol ( ) );
	alib::xmlApi < primitive::Integer >::compose ( output, object.getId ( ) );
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::END_ELEMENT );
}

} /* namespace alib */

namespace {

static auto valuePrinter = registration::ValuePrinterRegister < alib::UniqueObject > ( );

static auto xmlWrite = registration::XmlWriterRegister < alib::Object, alib::UniqueObject > ( );

static auto xmlReaded = registration::XmlReaderRegister < alib::Object, alib::UniqueObject > ( );

} /* namespace */
