/*
 * UniqueObject.h
 *
 *  Created on: Nov 11, 2016
 *      Author: Jan Travnicek
 */

#ifndef UNIQUE_OBJECT_H_
#define UNIQUE_OBJECT_H_

#include "Object.h"
#include <primitive/Integer.h>

namespace alib {

/**
 * Represents symbol in an alib.
 */
class UniqueObject final : public ObjectBase {
protected:
	Object m_symbol;
	primitive::Integer m_id;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit UniqueObject ( Object symbol, primitive::Integer id );

	virtual ObjectBase * clone ( ) const;

	virtual ObjectBase * plunder ( ) &&;

	/**
	 * @return name of the symbol
	 */
	const Object & getSymbol ( ) const;

	const primitive::Integer & getId ( ) const;

	primitive::Integer & getId ( );

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const UniqueObject & other ) const;

	virtual void operator >>( std::ostream & ) const;

	virtual explicit operator std::string ( ) const;

	virtual ObjectBase * inc ( ) &&;
};

} /* namespace alib */

namespace alib {

template < >
struct xmlApi < alib::UniqueObject > {
	static alib::UniqueObject parse ( ext::deque < sax::Token >::iterator & input );
	static bool first ( const ext::deque < sax::Token >::const_iterator & input );
	static const std::string & xmlTagName ( );
	static void compose ( ext::deque < sax::Token > & output, const alib::UniqueObject & data );
};

} /* namespace alib */

#endif /* UNIQUE_OBJECT_H_ */
