/*
 * Void.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef VOID_H_
#define VOID_H_

#include "ObjectBase.h"
#include <deque>
#include "../sax/Token.h"
#include "../core/xmlApi.hpp"

namespace alib {

/**
 * Represents void.
 */
class Void final : public ObjectBase {
public:
	/**
	 * Creates a blank symbol.
	 * @param symbol name of the symbol
	 */
	explicit Void ( );

	virtual ObjectBase * clone ( ) const;
	virtual ObjectBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const Void & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	static Void VOID;

	virtual ObjectBase * inc ( ) &&;
};

} /* namespace alib */

namespace alib {

template < >
struct xmlApi < void > {
	static void parse ( ext::deque < sax::Token >::iterator & input );
	static bool first ( const ext::deque < sax::Token >::const_iterator & input );
	static const std::string & xmlTagName ( );
	static void compose ( ext::deque < sax::Token > & output );
};

template < >
struct xmlApi < alib::Void > {
	static alib::Void parse ( ext::deque < sax::Token >::iterator & input );
	static bool first ( const ext::deque < sax::Token >::const_iterator & input );
	static const std::string & xmlTagName ( );
	static void compose ( ext::deque < sax::Token > & output, const alib::Void & data );
};

} /* namespace alib */

#endif /* VOID_H_ */
