/*
 * Bool.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#include "Bool.h"
#include <sax/FromXMLParserHelper.h>
#include <exception/CommonException.h>
#include "Primitive.h"
#include <object/Object.h>

#include <registration/ValuePrinterRegistration.hpp>
#include <registration/XmlRegistration.hpp>

namespace primitive {

Bool::Bool(bool data) : m_data(data) {

}

PrimitiveBase* Bool::clone() const {
	return new Bool(*this);
}

PrimitiveBase* Bool::plunder() && {
	return new Bool(std::move(*this));
}

bool Bool::getData() const {
	return m_data;
}

Bool::operator bool ( ) const {
	return m_data;
}

void Bool::setData(bool data) {
	this->m_data = data;
}

int Bool::compare(const Bool& other) const {
	return m_data - other.m_data;
}

void Bool::operator>>(std::ostream& out) const {
	out << "(Bool " << (m_data ? "true" : "false") << ")";
}

Bool::operator std::string() const {
	return m_data ? "true" : "false";
}

PrimitiveBase * Bool::inc() && {
	m_data = !m_data;
	return NULL;
}

} /* namespace primitive */

namespace alib {

bool xmlApi < bool >::parse ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );

	std::string tmp = sax::FromXMLParserHelper::popTokenData ( input, sax::Token::TokenType::CHARACTER );
	bool data;
	if ( tmp == "true" )
		data = true;
	else if ( tmp == "false" )
		data = false;
	else
		throw exception::CommonException ( "Invalid boolean value" );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, xmlTagName ( ) );

	return data;
}

bool xmlApi < bool >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

const std::string & xmlApi < bool >::xmlTagName ( ) {
	static std::string xmlTagName = "Bool";

	return xmlTagName;
}

void xmlApi < bool >::compose ( ext::deque < sax::Token > & output, bool data ) {
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::START_ELEMENT);
	if ( data )
		output.emplace_back ( "true", sax::Token::TokenType::CHARACTER );
	else
		output.emplace_back ( "false", sax::Token::TokenType::CHARACTER );
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::END_ELEMENT );
}

primitive::Bool xmlApi < primitive::Bool >::parse ( ext::deque < sax::Token >::iterator & input ) {
	return primitive::Bool ( xmlApi < bool >::parse ( input ) );
}

bool xmlApi < primitive::Bool >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

const std::string & xmlApi < primitive::Bool >::xmlTagName ( ) {
	return xmlApi < bool >::xmlTagName ( );
}

void xmlApi < primitive::Bool >::compose ( ext::deque < sax::Token > & output, const primitive::Bool & data ) {
	xmlApi < bool >::compose ( output, data.getData ( ) );
}

} /* namespace alib */

namespace {

static auto valuePrinter = registration::ValuePrinterRegister < primitive::Bool > ( );

static auto xmlWrite = registration::XmlWriterRegister < primitive::Primitive, primitive::Bool > ( );
static auto xmlWrite2 = registration::XmlWriterRegister < alib::Object, primitive::Bool > ( );

static auto xmlReader = registration::XmlReaderRegister < primitive::Primitive, primitive::Bool > ( );
static auto xmlReaded2 = registration::XmlReaderRegister < alib::Object, primitive::Bool > ( );

} /* namespace */
