/*
 * Character.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "Character.h"
#include <sax/FromXMLParserHelper.h>
#include "Primitive.h"
#include <object/Object.h>

#include <registration/ValuePrinterRegistration.hpp>
#include <registration/XmlRegistration.hpp>

namespace primitive {

Character::Character(char data) : m_data(data) {

}

PrimitiveBase* Character::clone() const {
	return new Character(*this);
}

PrimitiveBase* Character::plunder() && {
	return new Character(std::move(*this));
}

char Character::getData() const {
	return m_data;
}

Character::operator char ( ) const {
	return m_data;
}

void Character::setData(char data) {
	this->m_data = data;
}

int Character::compare(const Character& other) const {
	return m_data - other.m_data;
}

void Character::operator>>(std::ostream& out) const {
	out << "(Character " << m_data << ")";
}

Character::operator std::string() const {
	return std::string(1, m_data);
}

PrimitiveBase * Character::inc() && {
	m_data++;
	return NULL;
}

} /* namespace primitive */

namespace alib {

char xmlApi < char >::parse ( ext::deque < sax::Token >::iterator & input) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
	char data = sax::FromXMLParserHelper::popTokenData ( input, sax::Token::TokenType::CHARACTER ) [ 0 ];
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, xmlTagName ( ) );
	return data;
}

bool xmlApi < char >::first ( const ext::deque < sax::Token >::const_iterator & input) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

const std::string & xmlApi < char >::xmlTagName ( ) {
	static std::string xmlTagName = "Character";

	return xmlTagName;
}

void xmlApi < char >::compose ( ext::deque < sax::Token > & output, char data ) {
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::START_ELEMENT );
	output.emplace_back ( std::string ( 1, data ), sax::Token::TokenType::CHARACTER );
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::END_ELEMENT );
}

primitive::Character xmlApi < primitive::Character >::parse ( ext::deque < sax::Token >::iterator & input ) {
	return primitive::Character ( xmlApi < char >::parse ( input ) );
}

bool xmlApi < primitive::Character >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

const std::string & xmlApi < primitive::Character >::xmlTagName ( ) {
	return xmlApi < char >::xmlTagName ( );
}

void xmlApi < primitive::Character >::compose ( ext::deque < sax::Token > & output, const primitive::Character & data ) {
	xmlApi < char >::compose ( output, data.getData ( ) );
}

} /* namespace alib */

namespace {

static auto valuePrinter = registration::ValuePrinterRegister < primitive::Character > ( );

static auto xmlWrite = registration::XmlWriterRegister < primitive::Primitive, primitive::Character > ( );
static auto xmlWrite2 = registration::XmlWriterRegister < alib::Object, primitive::Character > ( );

static auto xmlReader = registration::XmlReaderRegister < primitive::Primitive, primitive::Character > ( );
static auto xmlReaded2 = registration::XmlReaderRegister < alib::Object, primitive::Character > ( );

} /* namespace */
