/*
 * Double.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#include "Double.h"
#include <sax/FromXMLParserHelper.h>
#include "Primitive.h"
#include <object/Object.h>

#include <registration/ValuePrinterRegistration.hpp>
#include <registration/XmlRegistration.hpp>

namespace primitive {

Double::Double(double data) : m_data(data) {

}

PrimitiveBase* Double::clone() const {
	return new Double(*this);
}

PrimitiveBase* Double::plunder() && {
	return new Double(std::move(*this));
}

double Double::getData() const {
	return m_data;
}

Double::operator double ( ) const {
	return m_data;
}

void Double::setData(double data) {
	this->m_data = data;
}

int Double::compare(const Double& other) const {
	if ( m_data < other.m_data )
		return -1;
	else if ( m_data > other.m_data )
		return 1;
	else
		return 0;
}

void Double::operator>>(std::ostream& out) const {
	out << "(Double " << m_data << ")";
}

Double::operator std::string() const {
	return ext::to_string(m_data);
}

PrimitiveBase * Double::inc() && {
	m_data += 1; // TODO fix this
	return this;
}

} /* namespace primitive */

namespace alib {

int xmlApi < double >::parse ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
	double data = ext::from_string < double > ( sax::FromXMLParserHelper::popTokenData ( input, sax::Token::TokenType::CHARACTER ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, xmlTagName ( ) );
	return data;
}

bool xmlApi < double >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

const std::string & xmlApi < double >::xmlTagName ( ) {
	static std::string xmlTagName = "Double";

	return xmlTagName;
}

void xmlApi < double >::compose ( ext::deque < sax::Token > & output, double data ) {
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::START_ELEMENT );
	output.emplace_back ( ext::to_string ( data ), sax::Token::TokenType::CHARACTER );
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::END_ELEMENT );
}

primitive::Double xmlApi < primitive::Double >::parse ( ext::deque < sax::Token >::iterator & input ) {
	return primitive::Double ( xmlApi < double >::parse ( input ) );
}

bool xmlApi < primitive::Double >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

const std::string & xmlApi < primitive::Double >::xmlTagName ( ) {
	return xmlApi < double >::xmlTagName ( );
}

void xmlApi < primitive::Double >::compose ( ext::deque < sax::Token > & output, const primitive::Double & data ) {
	xmlApi < double >::compose ( output, data.getData ( ) );
}

} /* namespace alib */

namespace {

static auto valuePrinter = registration::ValuePrinterRegister < primitive::Double > ( );

static auto xmlWrite = registration::XmlWriterRegister < primitive::Primitive, primitive::Double > ( );
static auto xmlWrite2 = registration::XmlWriterRegister < alib::Object, primitive::Double > ( );

static auto xmlReader = registration::XmlReaderRegister < primitive::Primitive, primitive::Double > ( );
static auto xmlReaded2 = registration::XmlReaderRegister < alib::Object, primitive::Double > ( );

} /* namespace */
