/*
 * Integer.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#include "Integer.h"
#include <sax/FromXMLParserHelper.h>
#include "Primitive.h"
#include <object/Object.h>

#include <registration/ValuePrinterRegistration.hpp>
#include <registration/XmlRegistration.hpp>

namespace primitive {

Integer::Integer(int data) : m_data(data) {

}

PrimitiveBase* Integer::clone() const {
	return new Integer(*this);
}

PrimitiveBase* Integer::plunder() && {
	return new Integer(std::move(*this));
}

int Integer::getData() const {
	return m_data;
}

Integer::operator int ( ) const {
	return m_data;
}

void Integer::setData(int data) {
	this->m_data = data;
}

int Integer::compare(const Integer& other) const {
	return m_data - other.m_data;
}

void Integer::operator>>(std::ostream& out) const {
	out << "(Integer " << m_data << ")";
}

Integer::operator std::string() const {
	return ext::to_string(m_data);
}

PrimitiveBase * Integer::inc() && {
	m_data++;
	return NULL;
}

} /* namespace primitive */

namespace alib {

int xmlApi < int >::parse ( ext::deque < sax::Token >::iterator & input) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
	int data = ext::from_string < int > ( sax::FromXMLParserHelper::popTokenData ( input, sax::Token::TokenType::CHARACTER ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, xmlTagName ( ) );
	return data;
}

bool xmlApi < int >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

const std::string & xmlApi < int >::xmlTagName ( ) {
	static std::string xmlTagName = "Integer";

	return xmlTagName;
}

void xmlApi < int >::compose ( ext::deque < sax::Token > & output, int data ) {
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::START_ELEMENT );
	output.emplace_back ( ext::to_string ( data ), sax::Token::TokenType::CHARACTER );
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::END_ELEMENT );
}

primitive::Integer xmlApi < primitive::Integer >::parse ( ext::deque < sax::Token >::iterator & input ) {
	return primitive::Integer ( xmlApi < int >::parse ( input ) );
}

bool xmlApi < primitive::Integer >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

const std::string & xmlApi < primitive::Integer >::xmlTagName ( ) {
	return xmlApi < int >::xmlTagName ( );
}

void xmlApi < primitive::Integer >::compose ( ext::deque < sax::Token > & output, const primitive::Integer & data ) {
	xmlApi < int >::compose ( output, data.getData ( ) );
}

} /* namespace alib */

namespace {

static auto valuePrinter = registration::ValuePrinterRegister < primitive::Integer > ( );

static auto xmlWrite = registration::XmlWriterRegister < primitive::Primitive, primitive::Integer > ( );
static auto xmlWrite2 = registration::XmlWriterRegister < alib::Object, primitive::Integer > ( );

static auto xmlReader = registration::XmlReaderRegister < primitive::Primitive, primitive::Integer > ( );
static auto xmlReaded2 = registration::XmlReaderRegister < alib::Object, primitive::Integer > ( );

} /* namespace */
