/*
 * String.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "String.h"
#include <sax/FromXMLParserHelper.h>
#include "Primitive.h"
#include <object/Object.h>

#include <registration/ValuePrinterRegistration.hpp>
#include <registration/XmlRegistration.hpp>

namespace primitive {

String::String(std::string data) : m_data(std::move(data)) {

}

String::String(const char* data) : String((std::string) data) {

}

PrimitiveBase* String::clone() const {
	return new String(*this);
}

PrimitiveBase* String::plunder() && {
	return new String(std::move(*this));
}

const std::string& String::getData() const {
	return m_data;
}

std::string& String::getData() {
	return m_data;
}

int String::compare(const String& other) const {
	return m_data.compare(other.m_data);
}

void String::operator>>(std::ostream& out) const {
	out << "(String " << m_data << ")";
}

String::operator std::string() const {
	return m_data;
}

PrimitiveBase * String::inc() && {
	m_data.push_back('\'');
	return NULL;
}

} /* namespace primitive */

namespace alib {

std::string xmlApi < std::string >::parse ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
	std::string data = "";
	if ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::CHARACTER ) )
		data = sax::FromXMLParserHelper::popTokenData ( input, sax::Token::TokenType::CHARACTER );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, xmlTagName ( ) );
	return data;
}

bool xmlApi < std::string >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

const std::string & xmlApi < std::string >::xmlTagName ( ) {
	static std::string xmlTagName = "String";

	return xmlTagName;
}

void xmlApi < std::string >::compose ( ext::deque < sax::Token > & output, const std::string & data) {
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::START_ELEMENT );
	output.emplace_back ( data, sax::Token::TokenType::CHARACTER );
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::END_ELEMENT );
}

primitive::String xmlApi < primitive::String >::parse ( ext::deque < sax::Token >::iterator & input ) {
	return primitive::String ( xmlApi < std::string >::parse ( input ) );
}

bool xmlApi < primitive::String >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

const std::string & xmlApi < primitive::String >::xmlTagName ( ) {
	return xmlApi < std::string >::xmlTagName ( );
}

void xmlApi < primitive::String >::compose ( ext::deque < sax::Token > & output, const primitive::String & data ) {
	xmlApi < std::string >::compose ( output, data.getData ( ) );
}

} /* namespace alib */

namespace {

static auto valuePrinter = registration::ValuePrinterRegister < primitive::String > ( );

static auto xmlWrite = registration::XmlWriterRegister < primitive::Primitive, primitive::String > ( );
static auto xmlWrite2 = registration::XmlWriterRegister < alib::Object, primitive::String > ( );

static auto xmlReader = registration::XmlReaderRegister < primitive::Primitive, primitive::String > ( );
static auto xmlReaded2 = registration::XmlReaderRegister < alib::Object, primitive::String > ( );

} /* namespace */
