/*
 * Unsigned.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#include "Unsigned.h"
#include <sax/FromXMLParserHelper.h>
#include "Primitive.h"
#include <object/Object.h>

#include <registration/ValuePrinterRegistration.hpp>
#include <registration/XmlRegistration.hpp>

namespace primitive {

Unsigned::Unsigned(unsigned data) : m_data(data) {

}

PrimitiveBase* Unsigned::clone() const {
	return new Unsigned(*this);
}

PrimitiveBase* Unsigned::plunder() && {
	return new Unsigned(std::move(*this));
}

unsigned Unsigned::getData() const {
	return m_data;
}

Unsigned::operator unsigned ( ) const {
	return m_data;
}

Unsigned::operator size_t ( ) const {
	return m_data;
}

void Unsigned::setData(unsigned data) {
	this->m_data = data;
}

int Unsigned::compare(const Unsigned& other) const {
	return m_data - other.m_data;
}

void Unsigned::operator>>(std::ostream& out) const {
	out << "(Unsigned " << m_data << ")";
}

Unsigned::operator std::string() const {
	return ext::to_string(m_data);
}

PrimitiveBase * Unsigned::inc() && {
	m_data++;
	return NULL;
}

} /* namespace primitive */

namespace alib {

unsigned xmlApi < unsigned >::parse ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
	unsigned data = ext::from_string < unsigned > ( sax::FromXMLParserHelper::popTokenData ( input, sax::Token::TokenType::CHARACTER ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, xmlTagName ( ) );
	return data;
}

bool xmlApi < unsigned >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

const std::string & xmlApi < unsigned >::xmlTagName ( ) {
	static std::string xmlTagName = "Unsigned";

	return xmlTagName;
}

void xmlApi < unsigned >::compose ( ext::deque < sax::Token > & output, unsigned data ) {
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::START_ELEMENT );
	output.emplace_back ( ext::to_string ( data ), sax::Token::TokenType::CHARACTER );
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::END_ELEMENT );
}

primitive::Unsigned xmlApi < primitive::Unsigned >::parse ( ext::deque < sax::Token >::iterator & input ) {
	return primitive::Unsigned ( xmlApi < unsigned >::parse ( input ) );
}

bool xmlApi < primitive::Unsigned >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

const std::string & xmlApi < primitive::Unsigned >::xmlTagName ( ) {
	return xmlApi < unsigned >::xmlTagName ( );
}

void xmlApi < primitive::Unsigned >::compose ( ext::deque < sax::Token > & output, const primitive::Unsigned & data ) {
	xmlApi < unsigned >::compose ( output, data.getData ( ) );
}

} /* namespace alib */

namespace {

static auto valuePrinter = registration::ValuePrinterRegister < primitive::Unsigned > ( );

static auto xmlWrite = registration::XmlWriterRegister < primitive::Primitive, primitive::Unsigned > ( );
static auto xmlWrite2 = registration::XmlWriterRegister < alib::Object, primitive::Unsigned > ( );

static auto xmlReader = registration::XmlReaderRegister < primitive::Primitive, primitive::Unsigned > ( );
static auto xmlReaded2 = registration::XmlReaderRegister < alib::Object, primitive::Unsigned > ( );

} /* namespace */
