/*
 * Unsigned.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_UNSIGNED_H_
#define PRIMITIVE_UNSIGNED_H_

#include "PrimitiveBase.h"
#include <core/xmlApi.hpp>

namespace primitive {

/**
 * Represents symbol in an alphabet.
 */
class Unsigned final : public PrimitiveBase {
protected:
	unsigned m_data;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit Unsigned ( unsigned data );

	virtual PrimitiveBase * clone ( ) const;

	virtual PrimitiveBase * plunder ( ) &&;

	/**
	 * @return name of the symbol
	 */
	unsigned getData ( ) const;

	explicit operator unsigned ( ) const;

	explicit operator size_t ( ) const;

	/**
	 * sets new content of primitive
	 */
	void setData ( unsigned data );

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const Unsigned & other ) const;

	virtual void operator >>( std::ostream & ) const;

	virtual explicit operator std::string ( ) const;

	virtual PrimitiveBase * inc ( ) &&;
};

} /* namespace primitive */

namespace alib {

template < >
struct xmlApi < unsigned > {
	static unsigned parse ( ext::deque < sax::Token >::iterator & input );
	static bool first ( const ext::deque < sax::Token >::const_iterator & input );
	static const std::string & xmlTagName ( );
	static void compose ( ext::deque < sax::Token > & output, unsigned data );
};

template < >
struct xmlApi < primitive::Unsigned > {
	static primitive::Unsigned parse ( ext::deque < sax::Token >::iterator & input );
	static bool first ( const ext::deque < sax::Token >::const_iterator & input );
	static const std::string & xmlTagName ( );
	static void compose ( ext::deque < sax::Token > & output, const primitive::Unsigned & data );
};

} /* namespace alib */

#endif /* PRIMITIVE_UNSIGNED_H_ */
