/*
 * UnsignedLong.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#include "UnsignedLong.h"
#include <sax/FromXMLParserHelper.h>
#include "Primitive.h"
#include <object/Object.h>

#include <registration/ValuePrinterRegistration.hpp>
#include <registration/XmlRegistration.hpp>

namespace primitive {

UnsignedLong::UnsignedLong(unsigned long data) : m_data(data) {

}

PrimitiveBase* UnsignedLong::clone() const {
	return new UnsignedLong(*this);
}

PrimitiveBase* UnsignedLong::plunder() && {
	return new UnsignedLong(std::move(*this));
}

unsigned long UnsignedLong::getData() const {
	return m_data;
}

UnsignedLong::operator unsigned long ( ) const {
	return m_data;
}

void UnsignedLong::setData(unsigned long data) {
	this->m_data = data;
}

int UnsignedLong::compare(const UnsignedLong& other) const {
	return m_data - other.m_data;
}

void UnsignedLong::operator>>(std::ostream& out) const {
	out << "(UnsignedLong " << m_data << ")";
}

UnsignedLong::operator std::string() const {
	return ext::to_string(m_data);
}

PrimitiveBase * UnsignedLong::inc() && {
	m_data++;
	return NULL;
}

} /* namespace primitive */

namespace alib {

unsigned long xmlApi < unsigned long >::parse ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
	unsigned long data = ext::from_string < unsigned long > ( sax::FromXMLParserHelper::popTokenData ( input, sax::Token::TokenType::CHARACTER ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, xmlTagName ( ) );
	return data;
}

bool xmlApi < unsigned long >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

const std::string & xmlApi < unsigned long >::xmlTagName ( ) {
	static std::string xmlTagName = "UnsignedLong";

	return xmlTagName;
}

void xmlApi < unsigned long >::compose ( ext::deque < sax::Token > & output, unsigned long data ) {
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::START_ELEMENT );
	output.emplace_back ( ext::to_string ( data ), sax::Token::TokenType::CHARACTER );
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::END_ELEMENT );
}

primitive::UnsignedLong xmlApi < primitive::UnsignedLong >::parse ( ext::deque < sax::Token >::iterator & input ) {
	return primitive::UnsignedLong ( xmlApi < unsigned long >::parse ( input ) );
}

bool xmlApi < primitive::UnsignedLong >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

const std::string & xmlApi < primitive::UnsignedLong >::xmlTagName ( ) {
	return xmlApi < unsigned long >::xmlTagName ( );
}

void xmlApi < primitive::UnsignedLong >::compose ( ext::deque < sax::Token > & output, const primitive::UnsignedLong & data ) {
	xmlApi < unsigned long >::compose ( output, data.getData ( ) );
}

} /* namespace alib */

namespace {

static auto valuePrinter = registration::ValuePrinterRegister < primitive::UnsignedLong > ( );

static auto xmlWrite = registration::XmlWriterRegister < primitive::Primitive, primitive::UnsignedLong > ( );
static auto xmlWrite2 = registration::XmlWriterRegister < alib::Object, primitive::UnsignedLong > ( );

static auto xmlReader = registration::XmlReaderRegister < primitive::Primitive, primitive::UnsignedLong > ( );
static auto xmlReaded2 = registration::XmlReaderRegister < alib::Object, primitive::UnsignedLong > ( );

} /* namespace */
