/*
 * UnsignedLong.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_UNSIGNED_LONG_H_
#define PRIMITIVE_UNSIGNED_LONG_H_

#include "PrimitiveBase.h"
#include <core/xmlApi.hpp>

namespace primitive {

/**
 * Represents symbol in an alphabet.
 */
class UnsignedLong final : public PrimitiveBase {
protected:
	unsigned long m_data;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit UnsignedLong ( unsigned long data );

	virtual PrimitiveBase * clone ( ) const;

	virtual PrimitiveBase * plunder ( ) &&;

	/**
	 * @return name of the symbol
	 */
	unsigned long getData ( ) const;

	explicit operator unsigned long ( ) const;

	/**
	 * sets new content of primitive
	 */
	void setData ( unsigned long data );

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const UnsignedLong & other ) const;

	virtual void operator >>( std::ostream & ) const;

	virtual explicit operator std::string ( ) const;

	virtual PrimitiveBase * inc ( ) &&;
};

} /* namespace primitive */

namespace alib {

template < >
struct xmlApi < unsigned long > {
	static unsigned long parse ( ext::deque < sax::Token >::iterator & input );
	static bool first ( const ext::deque < sax::Token >::const_iterator & input );
	static const std::string & xmlTagName ( );
	static void compose ( ext::deque < sax::Token > & output, unsigned long data );
};

template < >
struct xmlApi < primitive::UnsignedLong > {
	static primitive::UnsignedLong parse ( ext::deque < sax::Token >::iterator & input );
	static bool first ( const ext::deque < sax::Token >::const_iterator & input );
	static const std::string & xmlTagName ( );
	static void compose ( ext::deque < sax::Token > & output, const primitive::UnsignedLong & data );
};

} /* namespace alib */

#endif /* PRIMITIVE_UNSIGNED_LONG_H_ */
